	/* G N O K K I
	   A Linux/Unix toolset and driver for the Nokia 3110/3810/8110 mobiles.
	   Copyright (C) Hugh Blemings, 1999  Released under the terms of 
       the GNU GPL, see file COPYING for more details.
	
	   This file:  gnmon.c

	   A (presently rather crude) wrapper to make use of the protocol
	   handling functions contained in fbus.c/.h.  Ultimately this
	   code will expand to allow command line options etc. etc. :)
	   This code was modified from code that appears in the serial 
	   howto - see README for URLs.
    
	   Any ugly hacks are mine, not the author of the howto!, at the
	   risk of being overly apologetic, this code is just cobbled together
	   for now - more elegant work will follow :) */

#include	<termios.h>
#include	<stdio.h>
#include	<unistd.h>
#include	<fcntl.h>
#include	<sys/signal.h>
#include	<sys/types.h>
#include	<sys/time.h>

#include	"fbus.h"

#define BAUDRATE B115200

	/* Change HANDSET DEVICE to suit your serial port.  You must have
	   read/write permissions for this port. */
#define HANDSET_DEVICE "/dev/ttyS0"

	/* Signal handler for incoming characters. */
void signal_handler_IO_A (int status);   

int	fd;

int	main()
{
	int 				count, line_count;
	struct termios 		oldtio, newtio;

		/* You can play around with retrieving SIM card memory entries by
		   uncommenting this line and the lines further down... */
/*	int					sent_request = 0;*/

	struct sigaction 	saio;
	fd_set				readfs;
	int					maxfd;
	int					select_result;
	int					count41 = 0;
	struct timeval		timeout;

	unsigned char 		init_string[4] = {0x55, 0x55, 0x55, 0x55};

		/* The following code is pretty much verbatim for Serial howto
		   recommended reading... :) */

	fprintf (stdout, "Gnokii starting...\n");

		/* open the device to be non-blocking (read will return immediatly) */
	fd = open(HANDSET_DEVICE, O_RDWR | O_NOCTTY | O_NONBLOCK);
	if (fd <0) {
		perror("Error opening handset device! ");
		exit(-1);
	}
   		/* install the signal handler before making the device asynchronous */
  	saio.sa_handler = signal_handler_IO_A;
	sigemptyset(&saio.sa_mask);
	saio.sa_flags = 0;
	saio.sa_restorer = NULL;
	sigaction(SIGIO, &saio, NULL);
       
		/* allow the process to receive SIGIO */
	fcntl(fd, F_SETOWN, getpid());
		/* Make the file descriptor asynchronous (the manual page says only 
		   O_APPEND and O_NONBLOCK, will work with F_SETFL...) */
	fcntl(fd, F_SETFL, FASYNC);

	tcgetattr(fd, &oldtio); /* save current port settings */
		/* set new port settings for canonical input processing */
	newtio.c_cflag = BAUDRATE | CS8 | CLOCAL | CREAD;
	newtio.c_iflag = IGNPAR;
	newtio.c_oflag = 0;
	newtio.c_lflag = 0;
	newtio.c_cc[VMIN] = 1;
	newtio.c_cc[VTIME] = 0;
	tcflush(fd, TCIFLUSH);
	tcsetattr(fd, TCSANOW, &newtio);

		/* Initialise the code for the FBUS interface. */     
	FBUS_Initialise(fd);

		/* calculate maxfd for use in select call. */	
	maxfd = fd + 1;
 
	line_count = 0;

	fprintf (stdout, "Sending initialisation sequence to handset...");
	fflush(stdout);

		/* Send init string to handset, this is a bunch of 0x55
		   characters. */
	for (count = 0; count < 100; count ++) {
		usleep(1000);
		write(fd, init_string, 1);
	}

		/* Now send the 0x15 message, the exact purpose is not understood
		   but if not sent, link doesn't work. */
	FBUS_TX_Send0x15Message(0x11);
	
	fprintf (stdout, "Done.\n");

		/* loop while waiting for input. normally we would do something
          useful here */ 
	while(1) {
		FD_SET(fd, &readfs);
		timeout.tv_usec = 0;
		timeout.tv_sec = 2;

		select_result = select (maxfd, &readfs, NULL, NULL, &timeout);

			/* We send an 0x41 message every six seconds or so (the timeout
			   of the select call is used for this) - to request SMS
			   messages.  The 0x4a message is a keepalive. */
		if (select_result == 0) {
			FBUS_TX_Send0x4aMessage();
			count41 ++;
			if (count41 >= 3) {
				FBUS_TX_Send0x3fMessage();
				count41 = 0;
			}
			fflush(stdout);
		}

			/* If you uncomment the following, it will ask the phone
			   to dump the specified memory location.  */
		/*if (sent_request == 0 && FBUS_LinkOK == true) {
			FBUS_TX_Send0x43_RequestMemoryLocation(0x02, 0x01); 
			sent_request = -1;
		}*/
	}

	/* We never get here as no signal handling is provided yet. */	
	/* restore old port settings */
	tcsetattr(fd, TCSANOW, &oldtio);
	return (0);
}

	/* Signal handler, simply calls the FBUS_RX_StateMachine
	   code for each received character.  This will either dissapear
	   inside the driver or into a separate thread as part of the
	   fbus.c/.h code. */

void signal_handler_IO_A (int status)
{
	unsigned char buf[255];
	int		count,res;

	res = read(fd, buf, 255);

	for (count = 0; count < res ; count ++) {
		FBUS_RX_StateMachine(buf[count]);
	}
}
